module EMI_SoilHydrologyType_ExchangeMod
  !
  use shr_kind_mod                          , only : r8 => shr_kind_r8
  use shr_log_mod                           , only : errMsg => shr_log_errMsg
  use abortutils                            , only : endrun
  use clm_varctl                            , only : iulog
  use ExternalModelInterfaceDataMod         , only : emi_data_list, emi_data
  use ExternalModelIntefaceDataDimensionMod , only : emi_data_dimension_list_type
  use SoilHydrologyType                     , only : soilhydrology_type
  use EMI_Atm2LndType_Constants
  use EMI_CanopyStateType_Constants
  use EMI_ChemStateType_Constants
  use EMI_EnergyFluxType_Constants
  use EMI_SoilHydrologyType_Constants
  use EMI_SoilStateType_Constants
  use EMI_TemperatureType_Constants
  use EMI_WaterFluxType_Constants
  use EMI_WaterStateType_Constants
  use EMI_Filter_Constants
  use EMI_ColumnType_Constants
  use EMI_Landunit_Constants
  !
  implicit none
  !
  !
  public :: EMI_Pack_SoilHydrologyType_at_Column_Level_for_EM
  public :: EMI_Unpack_SoilHydrologyType_at_Column_Level_from_EM

contains
  
!-----------------------------------------------------------------------
  subroutine EMI_Pack_SoilHydrologyType_at_Column_Level_for_EM(data_list, em_stage, &
        num_filter, filter, soilhydrology_vars)
    !
    ! !DESCRIPTION:
    ! Pack data from ALM soilhydrology_vars for EM
    !
    ! !USES:
    use clm_varpar             , only : nlevsoi, nlevgrnd, nlevsno
    !
    implicit none
    !
    ! !ARGUMENTS:
    class(emi_data_list)     , intent(in) :: data_list
    integer                  , intent(in) :: em_stage
    integer                  , intent(in) :: num_filter
    integer                  , intent(in) :: filter(:)
    type(soilhydrology_type) , intent(in) :: soilhydrology_vars
    !
    ! !LOCAL_VARIABLES:
    integer                             :: fc,c,j
    class(emi_data), pointer            :: cur_data
    logical                             :: need_to_pack
    integer                             :: istage
    integer                             :: count

    associate(& 
         zwt      => soilhydrology_vars%zwt_col      , &
         qflx_bot => soilhydrology_vars%qflx_bot_col , &
         fracice  => soilhydrology_vars%fracice_col    &
         )

    count = 0
    cur_data => data_list%first
    do
       if (.not.associated(cur_data)) exit
       count = count + 1

       need_to_pack = .false.
       do istage = 1, cur_data%num_em_stages
          if (cur_data%em_stage_ids(istage) == em_stage) then
             need_to_pack = .true.
             exit
          endif
       enddo

       if (need_to_pack) then

          select case (cur_data%id)

          case (L2E_STATE_WTD)
             do fc = 1, num_filter
                c = filter(fc)
                cur_data%data_real_1d(c) = zwt(c)
             enddo
             cur_data%is_set = .true.

          case (L2E_STATE_QCHARGE)
             do fc = 1, num_filter
                c = filter(fc)
                cur_data%data_real_1d(c) = qflx_bot(c)
             enddo
             cur_data%is_set = .true.

          case (L2E_STATE_FRACICE)
             do fc = 1, num_filter
                c = filter(fc)
                do j = 1, nlevgrnd
                   cur_data%data_real_2d(c,j) = fracice(c,j)
                enddo
             enddo
             cur_data%is_set = .true.

          end select

       endif

       cur_data => cur_data%next
    enddo

    end associate

  end subroutine EMI_Pack_SoilHydrologyType_at_Column_Level_for_EM

!-----------------------------------------------------------------------
  subroutine EMI_Unpack_SoilHydrologyType_at_Column_Level_from_EM(data_list, em_stage, &
        num_filter, filter, soilhydrology_vars)
    !
    ! !DESCRIPTION:
    ! Unpack data for ALM soilhydrology_vars from EM
    !
    ! !USES:
    use clm_varpar             , only : nlevsoi, nlevgrnd, nlevsno
    !
    implicit none
    !
    ! !ARGUMENTS:
    class(emi_data_list)     , intent(in) :: data_list
    integer                  , intent(in) :: em_stage
    integer                  , intent(in) :: num_filter
    integer                  , intent(in) :: filter(:)
    type(soilhydrology_type) , intent(in) :: soilhydrology_vars
    !
    ! !LOCAL_VARIABLES:
    integer                             :: fc,c,j
    class(emi_data), pointer            :: cur_data
    logical                             :: need_to_pack
    integer                             :: istage
    integer                             :: count

    associate(& 
         zwt     => soilhydrology_vars%zwt_col     , &
         qcharge => soilhydrology_vars%qcharge_col   &
         )

    count = 0
    cur_data => data_list%first
    do
       if (.not.associated(cur_data)) exit
       count = count + 1

       need_to_pack = .false.
       do istage = 1, cur_data%num_em_stages
          if (cur_data%em_stage_ids(istage) == em_stage) then
             need_to_pack = .true.
             exit
          endif
       enddo

       if (need_to_pack) then

          select case (cur_data%id)

          case (E2L_STATE_WTD)
             do fc = 1, num_filter
                c = filter(fc)
                zwt(c) = cur_data%data_real_1d(c)
             enddo
             cur_data%is_set = .true.

          case (E2L_FLUX_AQUIFER_RECHARGE)
             do fc = 1, num_filter
                c = filter(fc)
                qcharge(c) = cur_data%data_real_1d(c)
             enddo
             cur_data%is_set = .true.

          end select

       endif

       cur_data => cur_data%next
    enddo

    end associate

  end subroutine EMI_Unpack_SoilHydrologyType_at_Column_Level_from_EM


end module EMI_SoilHydrologyType_ExchangeMod
